@@ -13,6 +13,7 @@ from PySide6.QtWidgets import (
     QVBoxLayout,
     QWidget,
 )
+
 from qasync import QEventLoop, asyncClose, asyncSlot
 
 
@@ -68,15 +69,16 @@ class MainWindow(QWidget):
 if __name__ == "__main__":
     app = QApplication(sys.argv)
 
-    event_loop = QEventLoop(app)
-    asyncio.set_event_loop(event_loop)
-
     app_close_event = asyncio.Event()
     app.aboutToQuit.connect(app_close_event.set)
-    
+
     main_window = MainWindow()
     main_window.show()
 
-    event_loop.create_task(main_window.boot())
-    event_loop.run_until_complete(app_close_event.wait())
-    event_loop.close()
+    async def async_main():
+        asyncio.create_task(main_window.boot())
+        await app_close_event.wait()
+
+    # for 3.11 or older use qasync.run instead of asyncio.run
+    # qasync.run(async_main())
+    asyncio.run(async_main(), loop_factory=QEventLoop)
