@@ -1,10 +1,11 @@
-import functools
 import asyncio
-import time
+import functools
 import sys
+import time
 
 # from PyQt6.QtWidgets import
 from PySide6.QtWidgets import QApplication, QProgressBar
+
 from qasync import QEventLoop, QThreadExecutor
 
 
@@ -34,8 +35,6 @@ def last_50(progress, loop):
 if __name__ == "__main__":
     app = QApplication(sys.argv)
 
-    event_loop = QEventLoop(app)
-    asyncio.set_event_loop(event_loop)
-
-    event_loop.run_until_complete(master())
-    event_loop.close()
+    # for 3.11 or older use qasync.run instead of asyncio.run
+    # qasync.run(master())
+    asyncio.run(master(), loop_factory=QEventLoop)
