@@ -4,19 +4,21 @@
 # BSD License
 
 import asyncio
-import logging
-import sys
-import os
 import ctypes
+import logging
 import multiprocessing
-from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
+import os
 import socket
 import subprocess
-
-import qasync
+import sys
+import threading
+import time
+from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
 
 import pytest
 
+import qasync
+
 
 @pytest.fixture
 def loop(request, application):
@@ -103,7 +105,7 @@ class TestCanRunTasksInExecutor:
             loop.run_until_complete(
                 asyncio.wait_for(
                     loop.run_in_executor(executor, self.blocking_failure),
-                    timeout=3.0,
+                    timeout=10.0,
                 )
             )
 
@@ -124,7 +126,7 @@ class TestCanRunTasksInExecutor:
     async def blocking_task(self, loop, executor, was_invoked):
         logging.debug("start blocking task()")
         fut = loop.run_in_executor(executor, self.blocking_func, was_invoked)
-        await asyncio.wait_for(fut, timeout=5.0)
+        await asyncio.wait_for(fut, timeout=10.0)
         logging.debug("start blocking task()")
 
 
@@ -138,7 +140,7 @@ def test_can_execute_subprocess(loop):
         await process.wait()
         assert process.returncode == 5
 
-    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=3))
+    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=10.0))
 
 
 def test_can_read_subprocess(loop):
@@ -158,7 +160,7 @@ def test_can_read_subprocess(loop):
         assert process.returncode == 0
         assert received_stdout.strip() == b"Hello async world!"
 
-    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=3))
+    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=10.0))
 
 
 def test_can_communicate_subprocess(loop):
@@ -179,7 +181,7 @@ def test_can_communicate_subprocess(loop):
         assert process.returncode == 0
         assert received_stdout.strip() == b"Hello async world!"
 
-    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=3))
+    loop.run_until_complete(asyncio.wait_for(mycoro(), timeout=10.0))
 
 
 def test_can_terminate_subprocess(loop):
@@ -587,6 +589,7 @@ def test_regression_bug13(loop, sock_pair):
             nonlocal result3
             assert result3 is None
             result3 = c_sock.recv(1)
+            loop._remove_reader(c_sock.fileno())
             client_done.set_result(True)
 
         loop._add_reader(c_sock.fileno(), cb1)
@@ -790,6 +793,130 @@ def test_not_running_immediately_after_stopped(loop):
     assert not loop.is_running()
 
 
+@pytest.mark.parametrize(
+    "async_wrap, expect_async_called, expect_exception",
+    [(False, False, True), (True, True, False)],
+)
+def test_async_wrap(
+    loop, application, async_wrap, expect_async_called, expect_exception
+):
+    """
+    Re-entering the event loop from a Task will fail if there is another
+    runnable task.
+    """
+    async_called = False
+    main_called = False
+
+    async def async_job():
+        nonlocal async_called
+        async_called = True
+
+    def sync_callback():
+        coro = async_job()
+        asyncio.create_task(coro)
+        assert not async_called
+        application.processEvents()
+        assert async_called if expect_async_called else not async_called
+        return 1, coro
+
+    async def main():
+        nonlocal main_called
+        if async_wrap:
+            res, coro = await qasync.asyncWrap(sync_callback)
+        else:
+            res, coro = sync_callback()
+            if expect_exception:
+                await coro  # avoid warnings about unawaited coroutines
+        assert res == 1
+        main_called = True
+        
+
+    exceptions = []
+    loop.set_exception_handler(lambda loop, context: exceptions.append(context))
+
+    loop.run_until_complete(main())
+    assert main_called, "The main function should have been called"
+
+    if expect_exception:
+        # We will now have an error in there, because the task 'async_job' could not
+        # be entered, because the task 'main' was still being executed by the event loop.
+        assert len(exceptions) == 1
+        assert isinstance(exceptions[0]["exception"], RuntimeError)
+    else:
+        assert len(exceptions) == 0
+
+
+def test_slow_callback_duration_logging(loop, caplog):
+    async def mycoro():
+        time.sleep(1)
+
+    caplog.clear()
+    loop.set_debug(True)
+    loop.slow_callback_duration = 0.1
+    with caplog.at_level(logging.WARNING):
+        loop.run_until_complete(mycoro())
+    assert len(caplog.records) == 1
+    msg = caplog.records[0].message
+    assert "Executing" in msg
+    assert "took" in msg
+    assert "seconds" in msg
+
+
+def test_run_until_complete_returns_future_result(loop):
+    async def coro():
+        await asyncio.sleep(0)
+        return 42
+
+    assert loop.run_until_complete(asyncio.wait_for(coro(), timeout=1)) == 42
+
+
+def test_run_forever_custom_exit_code(loop, application):
+    if hasattr(application, "exec"):
+        orig_exec = application.exec
+        application.exec = lambda: 42
+        try:
+            assert loop.run_forever() == 42
+        finally:
+            application.exec = orig_exec
+    else:
+        orig_exec = application.exec_
+        application.exec_ = lambda: 42
+        try:
+            assert loop.run_forever() == 42
+        finally:
+            application.exec_ = orig_exec
+
+
+def test_qeventloop_in_qthread():
+    class CoroutineExecutorThread(qasync.QtCore.QThread):
+        def __init__(self, coro):
+            super().__init__()
+            self.coro = coro
+            self.loop = None
+
+        def run(self):
+            self.loop = qasync.QEventLoop(self)
+            asyncio.set_event_loop(self.loop)
+            asyncio.run(self.coro)
+
+        def join(self):
+            self.loop.stop()
+            self.loop.close()
+            self.wait()
+
+    event = threading.Event()
+
+    async def coro():
+        await asyncio.sleep(0.1)
+        event.set()
+
+    thread = CoroutineExecutorThread(coro())
+    thread.start()
+
+    assert event.wait(timeout=1), "Coroutine did not execute successfully"
+
+    thread.join()  # Ensure thread cleanup
+
 def teardown_module(module):
     """
     Remove handlers from all loggers
